@@ -15,6 +15,8 @@
 
 from __future__ import absolute_import
 
+from robot.errors import RobotError
+
 try:
     import configparser # type: ignore
 except:
@@ -22,13 +24,13 @@ except:
     # Support Python 2
 
 from robot.libraries.BuiltIn import BuiltIn
-from robotremoteserver import RobotRemoteServer
+from .robotremoteserver import RobotRemoteServer
 from robot.libraries.Remote import Remote
 from robot.running import TestLibrary
 from robot.api import logger
 import threading
 import time
-from typing import List, Dict, Tuple, Union, Optional
+from typing import List, Dict, Tuple, Set, Optional, Any
 
 PABOT_LAST_LEVEL = "PABOTLASTLEVEL"
 PABOT_QUEUE_INDEX = "PABOTQUEUEINDEX"
@@ -39,15 +41,17 @@ class _PabotLib(object):
 
     _TAGS_KEY = "tags"
 
-    def __init__(self, resourcefile=None):
+    def __init__(self, resourcefile=None): # type: (Optional[str]) -> None
         self._locks = {} # type: Dict[str, Tuple[str, int]]
         self._owner_to_values = {} # type: Dict[str, Dict[str, object]]
         self._parallel_values = {} # type: Dict[str, object]
-        self._remote_libraries = {} # type: Dict[str, object]
+        self._remote_libraries = {} # type: Dict[str, Tuple[int, RobotRemoteServer, threading.Thread]]
         self._values = self._parse_values(resourcefile)
+        self._added_suites = [] # type: List[Tuple[str, List[str]]]
+        self._ignored_executions = set() # type: Set[str]
 
-    def _parse_values(self, resourcefile):
-        vals = {} # type: Dict[str, Dict[str, any]]
+    def _parse_values(self, resourcefile): # type: (Optional[str]) -> Dict[str, Dict[str, Any]]
+        vals = {} # type: Dict[str, Dict[str, Any]]
         if resourcefile is None:
             return vals
         conf = configparser.ConfigParser()
@@ -62,13 +66,13 @@ class _PabotLib(object):
                 vals[section][self._TAGS_KEY] = []
         return vals
 
-    def set_parallel_value_for_key(self, key, value):
+    def set_parallel_value_for_key(self, key, value): # type: (str, object) -> None
         self._parallel_values[key] = value
 
-    def get_parallel_value_for_key(self, key):
+    def get_parallel_value_for_key(self, key): # type: (str) -> object
         return self._parallel_values.get(key, "")
 
-    def acquire_lock(self, name, caller_id):
+    def acquire_lock(self, name, caller_id): # type: (str, str) -> bool
         if name in self._locks and caller_id != self._locks[name][0]:
             return False
         if name not in self._locks:
@@ -76,7 +80,7 @@ class _PabotLib(object):
         self._locks[name] = (caller_id, self._locks[name][1] + 1)
         return True
 
-    def release_lock(self, name, caller_id):
+    def release_lock(self, name, caller_id): # type: (str, str) -> None
         assert self._locks[name][0] == caller_id
         self._locks[name] = (caller_id, self._locks[name][1] - 1)
         if self._locks[name][1] == 0:
@@ -110,21 +114,23 @@ class _PabotLib(object):
         # and the caller needs to wait until one is free.
         return (None, None)
 
-    def release_value_set(self, caller_id):
-        self._owner_to_values[caller_id] = None
+    def release_value_set(self, caller_id): # type: (str) -> None
+        if caller_id not in self._owner_to_values:
+            return
+        del self._owner_to_values[caller_id]
 
-    def disable_value_set(self, setname, caller_id):
-        self._owner_to_values[caller_id] = None
+    def disable_value_set(self, setname, caller_id): # type: (str, str) -> None
+        del self._owner_to_values[caller_id]
         del self._values[setname]
 
-    def get_value_from_set(self, key, caller_id):
+    def get_value_from_set(self, key, caller_id): # type: (str, str) -> object
         if caller_id not in self._owner_to_values:
             raise AssertionError('No value set reserved for caller process')
         if key not in self._owner_to_values[caller_id]:
             raise AssertionError('No value for key "%s"' % key)
         return self._owner_to_values[caller_id][key]
 
-    def import_shared_library(self, name):
+    def import_shared_library(self, name): # type: (str) -> int
         if name in self._remote_libraries:
             return self._remote_libraries[name][0]
         imported = TestLibrary(name)
@@ -136,6 +142,20 @@ class _PabotLib(object):
         self._remote_libraries[name] = (port, server, server_thread)
         return port
 
+    def add_suite_to_execution_queue(self, suitename, variables):  # type: (str, List[str]) -> None
+        self._added_suites.append((suitename, variables or []))
+
+    def get_added_suites(self): # type: () -> List[Tuple[str, List[str]]]
+        added_suites = self._added_suites
+        self._added_suites = []
+        return added_suites
+
+    def ignore_execution(self, caller_id): # type: (str) -> None
+        self._ignored_executions.add(caller_id)
+
+    def is_ignored_execution(self, caller_id): # type: (str) -> bool
+        return caller_id in self._ignored_executions
+
     def stop_remote_libraries(self):
         for name in self._remote_libraries:
             self._remote_libraries[name][1].stop_remote_server()
@@ -143,7 +163,6 @@ class _PabotLib(object):
             self._remote_libraries[name][2].join()
 
 
-
 class PabotLib(_PabotLib):
 
     __version__ = 0.67
@@ -193,10 +212,10 @@ class PabotLib(_PabotLib):
         try:
             self.release_locks()
             self.release_value_set()
-        except RuntimeError:
+        except RuntimeError as err:
             # This is just last line of defence
             # Ignore connection errors if library server already closed
-            logger.console("pabot.PabotLib#_close: threw an exception: is --pabotlib flag used?", stream='stderr')
+            logger.console("pabot.PabotLib#_close: threw an exception: is --pabotlib flag used? ErrorDetails: {0}".format(repr(err)), stream='stderr')
             pass
 
     @property
@@ -334,12 +353,15 @@ class PabotLib(_PabotLib):
         if self._remotelib:
             try:
                 return self._remotelib.run_keyword(keyword, args, {})
-            except RuntimeError:
-                logger.error('No connection - is pabot called with --pabotlib option?')
+            except RuntimeError as err:
+                logger.error("RuntimeError catched in remotelib keyword execution. Maybe there is no connection - is pabot called with --pabotlib option? ErrorDetails: {0}".format(repr(err)))
                 self.__remotelib = None
                 raise
         return getattr(_PabotLib, keyword)(self, *args)
 
+    def add_suite_to_execution_queue(self, suitename, variables=None):
+        self._run_with_lib('add_suite_to_execution_queue', suitename, variables)
+
     def get_parallel_value_for_key(self, key):
         """
         Get the value for a key. If there is no value for the key then empty
@@ -360,8 +382,8 @@ class PabotLib(_PabotLib):
                     time.sleep(PabotLib._pollingSeconds)
                     if PabotLib._polling_logging: logger.debug('waiting for lock to release')
                 return True
-            except RuntimeError:
-                logger.error('No connection - is pabot called with --pabotlib option?')
+            except RuntimeError as err:
+                logger.error("RuntimeError catched in remote acquire_lock execution. Maybe there is no connection - is pabot called with --pabotlib option? ErrorDetails: {0}".format(repr(err)))
                 self.__remotelib = None
                 raise
         return _PabotLib.acquire_lock(self, name, self._my_id)
@@ -401,8 +423,8 @@ class PabotLib(_PabotLib):
                         return self._setname
                     time.sleep(PabotLib._pollingSeconds)
                     if PabotLib._polling_logging: logger.debug('waiting for a value set')
-            except RuntimeError:
-                logger.error('No connection - is pabot called with --pabotlib option?')
+            except RuntimeError as err:
+                logger.error("RuntimeError catched in remote _acquire_value_set execution. Maybe there is no connection - is pabot called with --pabotlib option? ErrorDetails: {0}".format(repr(err)))
                 self.__remotelib = None
                 raise
         self._setname, self._valueset = _PabotLib.acquire_value_set(self, self._my_id, *tags)
@@ -420,6 +442,13 @@ class PabotLib(_PabotLib):
             raise AssertionError('No value for key "%s"' % key)
         return self._valueset[key]
 
+    def ignore_execution(self):
+        self._run_with_lib('ignore_execution', self._my_id)
+        error = RobotError('Ignore')
+        error.ROBOT_EXIT_ON_FAILURE = True
+        error.ROBOT_CONTINUE_ON_FAILURE = False
+        raise error
+
     def release_value_set(self):
         """
         Release a reserved value set so that other executions can use it also.
